searchData={"content_type":"text/markdown","items":[{"doc":"","ref":"Approval.html","title":"Approval","type":"module"},{"doc":"This package provides some utilities to help automatically generate\nreference images for test cases, review those reference images\nand compare snapshots to the reference images to look for regressions.","ref":"Approval.html#module-lightweight-approval-testing-for-elixir","title":"Lightweight approval testing for Elixir - Approval","type":"module"},{"doc":"Sets up an approval test.\n\nTakes the following arguments:\n\n  - `:snapshot` - the data generated by your code.\n    If the data represents an image, it should be given as\n    `File.read!(some_path)`.\n\n  - `:reference` - the reference data with which the snapshots\n    will be compared. Again, if it represents an image, it should\n    be given as `File.read!(some_path)`. This file doesn't need\n    to exist. In fact, the whole point is that the `approval` macro\n    will generate it the first time the test is run and you'll only\n    have to review it later. If you already have a reference file,\n    it won't be overwritten.\n\n  - `:reviewed` (*optional*, default: `false`) - whether the reference\n    data has been reviewed by the user or not\n\nFirst, the test wouldn't be manually reviewed:\n\n    import Approve\n\n    test \"example test\" do\n      snapshot = File.write!(\"snapshot.png\", create_new_image_data(...))\n\n      approve snapshot: File.read!(\"snapshot.png\"),\n              reference: File.read!(\"reference.png\"), # this file doesn't exist, it will be created\n              reviewed: false # the user hasn't manually reviewed the reference yet\n    end\n\nWhen you run the test above, the code will copy the snapshot into the reference.\nThis creates the new `\"reference.png\"` file, which you must review manually.\nFurther snapshots will be compared to this reference.\nUntil you review the reference file, the test will fail.\nAfter reviewing the reference file, you should update the code above by replacing\n`reviewed: false` by `reviewed: true`, so that it reads:\n\n    import Approve\n\n    test \"example test\" do\n      snapshot = File.write!(\"snapshot.png\", create_new_image_data(...))\n\n      approve snapshot: File.read!(\"snapshot.png\"),\n              reference: File.read!(\"reference.png\"), # this file exists and has been reviewed\n              reviewed: true # the user has reviewed the reference\n    end\n\nIf at any point the snapshot file becomes different from the reference,\nthe test will fail and the `approve` macro will generate an HTML file\nnext to the snapshot named \"snapshot.png.diff.html\" (in general it will be\n`reference_path <> \"diff.html\"`). That HTML file contains the old and new\nversions of the images side by side. In the middle it shows the difference\nbetween the two images.\n\nIf at any point you want to change the reference file (let's say you have\nfound a bug in the code that generated the previous reference), you can\nsimply add a new reference file manually.\n\nThis is an example `diff.html` file made from two images:\n\n![diff.html](assets/diff_html.png)","ref":"Approval.html#approve/1","title":"Approval.approve/1","type":"macro"},{"doc":"The most portable way of comparing two images is by running javascript\ninside a web browser, which is what we do here.\nOne possible criticism is that the user needs to manually open the HTML\nfiles in a web browser to view the results, instead of having the test suite\nspin up a web app which would centralize all images and even allow one to\napprove changes from the web browser itself.\n\nHowever, that implementation is not very portable and is quite complex.\nThis implementation is optimized for simplicity, and it has settled on using\nthe filesystem as the only source of persistence.\nThe downside is the whole manual fiddling with files, but that is not too bad\nin practice\n\nIn the future, I plan to support other kinds of data besides images,\nand that is the goal of the somewhat verbose `snapshot: File.read!(path)`\nsyntax, instead of the shorter `snapshot: path` alternative.","ref":"Approval.html#approve/1-implementation-notes","title":"Implementation notes - Approval.approve/1","type":"macro"},{"doc":"","ref":"Approval.ApprovalError.html","title":"Approval.ApprovalError","type":"exception"},{"doc":"# Approval\n\nSupport for approval testing, especially of code that generated image.\n\nFirst, the test wouldn't be manually reviewed:\n\n    import Approve\n\n    test \"example test\" do\n      snapshot = File.write!(\"snapshot.png\", create_new_image_data(...))\n\n      approve snapshot: File.read!(\"snapshot.png\"),\n              reference: File.read!(\"reference.png\"), # this file doesn't exist, it will be created\n              reviewed: false # the user hasn't manually reviewed the reference yet\n    end\n\nWhen you run the test above, the code will copy the snapshot into the reference.\nThis creates the new `\"reference.png\"` file, which you must review manually.\nFurther snapshots will be compared to this reference.\nUntil you review the reference file, the test will fail.\nAfter reviewing the reference file, you should update the code above by replacing\n`reviewed: false` by `reviewed: true`, so that it reads:\n\n    import Approve\n\n    test \"example test\" do\n      snapshot = File.write!(\"snapshot.png\", create_new_image_data(...))\n\n      approve snapshot: File.read!(\"snapshot.png\"),\n              reference: File.read!(\"reference.png\"), # this file exists and has been reviewed\n              reviewed: true # the user has reviewed the reference\n    end\n\nIf at any point the snapshot file becomes different from the reference,\nthe test will fail and the `approve` macro will generate an HTML file\nnext to the snapshot named \"snapshot.png.diff.html\" (in general it will be\n`reference_path <> \"diff.html\"`). That HTML file contains the old and new\nversions of the images side by side. In the middle it shows the difference\nbetween the two images.\n\nIf at any point you want to change the reference file (let's say you have\nfound a bug in the code that generated the previous reference), you can\nsimply add a new reference file manually.\n\nThis is an example `diff.html` file made from two images:\n\n![diff.html](assets/diff_html.png)","ref":"readme.html","title":"Approval","type":"extras"},{"doc":"The most portable way of comparing two images is by running javascript\ninside a web browser, which is what we do here.\nOne possible criticism is that the user needs to manually open the HTML\nfiles in a web browser to view the results, instead of having the test suite\nspin up a web app which would centralize all images and even allow one to\napprove changes from the web browser itself.\n\nHowever, that implementation is not very portable and is quite complex.\nThis implementation is optimized for simplicity, and it has settled on using\nthe filesystem as the only source of persistence.\nThe downside is the whole manual fiddling with files, but that is not too bad\nin practice\n\nIn the future, I plan to support other kinds of data besides images,\nand that is the goal of the somewhat verbose `snapshot: File.read!(path)`\nsyntax, instead of the shorter `snapshot: path` alternative.","ref":"readme.html#implementation-notes","title":"Implementation notes - Approval","type":"extras"},{"doc":"The package is [available in Hex](https://hex.pm/docs/publish) and can be installed\nby adding `approval` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:approval, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThe docs can be found at  .","ref":"readme.html#installation","title":"Installation - Approval","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}